<?php
/**
 * @package Zo2Framework for Joomla! 2.5
 * @author http://www.ZooTemplate.com
 * @copyright(C) 2010- ZooTemplate.com
 * @license PHP files are GNU/GPL
**/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');
 
// import Joomla modelform library
jimport('joomla.application.component.modeladmin');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');
jimport('joomla.html.parameter');
jimport('joomla.form.form');

/**
 * Feed Model
 */
class Zo2FrameworkModelTemplate extends JModelAdmin
{
	var $template 	= NULL;
	var $theme_path	= NULL;
	var $db			= NULL;
	
	public function __construct($config = array())
	{
		parent::__construct($config);
		
		$this->theme_path 	= JPATH_ROOT . DS . "templates";
		$this->db			= &JFactory::getDBO();
	}
				
	
	protected function allowEdit($data = array(), $key = 'id')
	{
		// Check specific edit permission then general edit permission.
		return JFactory::getUser()->authorise('core.edit', 'com_zo2framework.message.'.((int) isset($data[$key]) ? $data[$key] : 0)) or parent::allowEdit($data, $key);
	}


	public function getTable($type = 'Template', $prefix = 'Zo2FrameworkTable', $config = array()) 
	{
		return JTable::getInstance($type, $prefix, $config);
	}
	
	
	public function getForm($data = array(), $loadData = true) 
	{
		// Get data
		$data 	= $this->loadFormData();
		$xfile	= $this->theme_path . DS . $data->theme . DS . "config.xml";
		
		// Get the form.
		if(JFile::exists($xfile)) {
			$form = new JForm('com_zo2framework.template');
			$form->loadFile($xfile);
			$this->preprocessForm($form, $data);
			$form->bind($data);
		}
		else {
			$form = $this->loadForm('com_zo2framework.template', 'template', array('control' => 'jform', 'load_data' => $loadData));
		}
		
		// Return false if empty
		if(empty($form)) {
			return false;
		}
		
		return $form;
	}
	
	
	protected function loadFormData() 
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_zo2framework.edit.template.data', array());
		if(empty($data)) {
			$data = $this->getItem();
		}
		foreach($data->params as $k=>$v) {
			$data->$k = $v;
		}
		return $data;
	}
	
	
	public function save()
	{
		if(JRequest::getVar('jform', null, 'post', 'none', 4)) {
			$_post = JRequest::get('post', 4);
			$post  = $_post['jform'];
			$post['header_text'] = (strlen($_POST['jform']['header_text']) > 2) ? base64_encode($_POST['jform']['header_text']) : '';
			$post['footer_text'] = (strlen($_POST['jform']['footer_text']) > 2) ? base64_encode($_POST['jform']['footer_text']) : '';
		}
		else {
			$post  = JRequest::get('post', 4);
			$post['header_text'] = (strlen($_POST['header_text']) > 2) ? base64_encode($_POST['header_text']) : '';
			$post['footer_text'] = (strlen($_POST['footer_text']) > 2) ? base64_encode($_POST['footer_text']) : '';
		}
		
		$id	  	= $post['id'];
		$published	= $post['published'];
		$params		= "";
		
		//Check unpublished default template
		if($post['home'] == 1 && $published == 0) {
			$this->setError(JTEXT::_('COM_ZO2FRAMEWORK_CANNOT_UNPUBLISHED_DEFAULT_TEMPLATE'));
			return false;
		}
		
		//Remove unused variable
		unset($post['id']);
		unset($post['home']);
		unset($post['published']);
		unset($post['task']);
		unset($post['option']);
		
		// Bind params
		$txt = array();
		foreach($post as $k => $v) {
			if(is_array($v)){
				$v 		= (count($v)) ? '["' . implode('","', $v) . '"]' : '[]';
				$txt[] 	= '"'.$k.'":' . $v;
			}
			else {
				$txt[] = '"'.$k.'":"' . $v . '"';
			}
		}
		$params = "{" . implode(",", $txt) . "}";
		
		//Store data
		parent::save(array('id'=>$id, 'params'=>$params, 'published'=>$published));
		
		return true;
	}
	
	
	/**
	 * Install an extension from either folder, url or upload.
	 *
	 * @return	boolean result of install
	 * @since	1.5
	 */
	function install()
	{
		$this->setState('action', 'install');

		// Set FTP credentials, if given.
		JClientHelper::setCredentialsFromRequest('ftp');
		$app = JFactory::getApplication();

		$package = $this->_getPackageFromUpload();

		// Was the package unpacked?
		if(!$package) {
			$app->setUserState('com_zo2framework.message', JText::_('COM_ZO2FRAMEWORK_UNABLE_TO_FIND_INSTALL_PACKAGE'));
			return false;
		}

		// Get an installer instance
		$installer = JInstaller::getInstance();

		// Install the package
		if(!$installer->install($package['dir'])) {
			// There was an error installing the package
			$msg = JText::_('COM_ZO2FRAMEWORK_INSTALL_ERROR');
			$result = false;
		} else {
			// Package installed sucessfully
			$msg = JText::_('COM_ZO2FRAMEWORK_INSTALL_SUCCESS');
			$result = true;
		}

		// Set some model state values
		$app	= JFactory::getApplication();
		$app->enqueueMessage($msg);
		$this->setState('name', $installer->get('name'));
		$this->setState('result', $result);
		$app->setUserState('com_zo2framework.message', $installer->message);
		$app->setUserState('com_zo2framework.extension_message', $installer->get('extension_message'));
		$app->setUserState('com_zo2framework.redirect_url', 'index.php?option=com_zo2framework&view=templates');

		// Cleanup the install files
		if(!is_file($package['packagefile'])) {
			$config = JFactory::getConfig();
			$package['packagefile'] = $config->get('tmp_path') . '/' . $package['packagefile'];
		}

		JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);


		return $result;
	}

	/**
	 * Works out an installation package from a HTTP upload
	 *
	 * @return package definition or false on failure
	 */
	protected function _getPackageFromUpload()
	{
		// Get the uploaded file information
		$userfile = JRequest::getVar('install_package', null, 'files', 'array');

		// Make sure that file uploads are enabled in php
		if(!(bool) ini_get('file_uploads')) {
			JError::raiseWarning('', JText::_('COM_ZO2FRAMEWORK_MSG_INSTALL_WARNINSTALLFILE'));
			return false;
		}

		// Make sure that zlib is loaded so that the package can be unpacked
		if(!extension_loaded('zlib')) {
			JError::raiseWarning('', JText::_('COM_ZO2FRAMEWORK_MSG_INSTALL_WARNINSTALLZLIB'));
			return false;
		}

		// If there is no uploaded file, we have a problem...
		if(!is_array($userfile)) {
			JError::raiseWarning('', JText::_('COM_ZO2FRAMEWORK_MSG_INSTALL_NO_FILE_SELECTED'));
			return false;
		}

		// Check if there was a problem uploading the file.
		if($userfile['error'] || $userfile['size'] < 1) {
			JError::raiseWarning('', JText::_('COM_ZO2FRAMEWORK_MSG_INSTALL_WARNINSTALLUPLOADERROR'));
			return false;
		}

		// Build the appropriate paths
		$config		= JFactory::getConfig();
		$tmp_dest	= $config->get('tmp_path') . '/' . $userfile['name'];
		$tmp_src	= $userfile['tmp_name'];

		// Move uploaded file
		jimport('joomla.filesystem.file');
		$uploaded = JFile::upload($tmp_src, $tmp_dest);

		// Unpack the downloaded package file
		$package = JInstallerHelper::unpack($tmp_dest);

		return $package;
	}
}