<?php
/**
 * @package Zo2Framework for Joomla! 2.5
 * @author http://www.ZooTemplate.com
 * @copyright(C) 2010- ZooTemplate.com
 * @license PHP files are GNU/GPL
**/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// import the Joomla modellist library
jimport('joomla.application.component.modellist');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');
jimport('joomla.utilities.simplexml');
jimport('joomla.base.adapter');

/**
 * FeedList Model
 */
class Zo2FrameworkModelTemplates extends JModelList
{
	
	var $prefix 	= NULL;
	var $theme_path	= NULL;
	var $db			= NULL;
	
	public function __construct($config = array())
	{
		parent::__construct($config);
		
		$this->db = &JFactory::getDBO();
		$_params  = $this->loadSystemConfig();
		$prefix   = $_params->get('template_prefix');
		
		$this->prefix		= ($prefix != null) ? explode(",", $prefix) : array("zt_", "mt_");
		$this->theme_path 	= JPATH_ROOT . DS . "templates";
		
		$this->reloadThemes();
	}
	
	public function loadSystemConfig()
	{
		$query = "SELECT params FROM #__zof_config WHERE id = 1";
		$this->db->setQuery($query);
		$row   = $this->db->loadResult();
		
		$params = new JParameter();
		$params->bind($row);
		
		return $params;
	}
	
	public function reloadThemes()
	{
		$templates = JFolder::folders($this->theme_path);
		$themes	   = array();
		
		//Insert new template
		if(count($templates))
		for($i = 0; $i < count($templates); $i++) {
			
			// Check prefix
			$tname = $templates[$i];
			$pos   = false;
			
			for($k = 0; $k < count($this->prefix); $k++) {
				$pos = strpos($tname, trim($this->prefix[$k]));
				if($pos !== false) break;
			}
			
			if($pos === false) continue;
			
			$query = "SELECT COUNT(*) FROM #__zof_themecfg WHERE theme = " . $this->db->Quote($tname);
			$this->db->setQuery($query);
			$exist = $this->db->loadResult();
			
			// Get param default
			$params = "{}";
			$xfile  = (JFile::exists($this->theme_path . DS . $tname . DS. "config.xml")) 
					? $this->theme_path . DS . $tname . DS. "config.xml"
					: dirname(__FILE__) . DS . "forms" . DS . "template.xml";
			
			if(JFile::exists($xfile)) {
				$params = $this->getParams($xfile);
			}
			
			if(!$exist) {
				$query = "INSERT INTO #__zof_themecfg VALUES('', " . $this->db->Quote($tname) . ", 0, '{$params}', 1)";
				$this->db->setQuery($query);
				$this->db->query();
			}
			
			$themes[] = $tname;
		}
		
		//Remove template not found.
		$query = "SELECT theme FROM #__zof_themecfg";
		$this->db->setQuery($query);
		$rows  = $this->db->loadResultArray();
		if(count($rows))
		for($i = 0; $i < count($rows); $i++) {
			if(!in_array($rows[$i], $themes)) {
				$query = "DELETE FROM `#__zof_themecfg` WHERE theme = " . $this->db->Quote($rows[$i]);
				$this->db->setQuery($query);
				$this->db->query();
			}
		}
	}			
	
	public function getParams($file)
	{
		$xml = JFactory::getXML($file);
		
		// Validate that we have a fieldset to use
		if (!isset($xml->fieldset))
		{
			return '{}';
		}
		
		// Getting the fieldset tags
		$fieldsets = $xml->fieldset;

		// Creating the data collection variable:
		$ini = array();

		// Iterating through the fieldsets:
		foreach ($fieldsets as $fieldset)
		{
			if (!count($fieldset->children()))
			{
				// Either the tag does not exist or has no children therefore we return zero files processed.
				return null;
			}

			// Iterating through the fields and collecting the name/default values:
			foreach ($fieldset as $field)
			{
				// Check against the null value since otherwise default values like "0"
				// cause entire parameters to be skipped.

				if (($name = $field->attributes()->name) === null)
				{
					continue;
				}

				if (($value = $field->attributes()->default) === null)
				{
					continue;
				}

				$ini[(string) $name] = (string) $value;
			}
		}

		return json_encode($ini);
	}
	
	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication('administrator');
		
		// Load the filter search.
		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);
		
		// List state information.
		parent::populateState('id', 'ASC');
	}
	
	
	protected function getListQuery()
	{
		// Create a new query object.		
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		// Select some fields
		$query->select('*');
		// From the hello table
		$query->from('#__zof_themecfg');
		
		if($this->getState('filter.search') !== '')
		{
			$token = $db->Quote('%'.$db->escape($this->getState('filter.search')).'%');
			
			// Compile the different search clauses.
			$searches	= array();
			$searches[]	= 'theme  LIKE ' . $token;

			// Add the clauses to the query.
			$query->where('('.implode(' OR ', $searches).')');
		}
		
		return $query;
	}
	
	public function reloadDefault()
	{
		$query = "SELECT template FROM #__template_styles WHERE client_id = 0 AND home = 1";
		$this->db->setQuery($query);
		$template = $this->db->loadResult();
		
		$query = "UPDATE `#__zof_themecfg` SET `home` = 1 WHERE `theme` = " . $this->db->Quote($template);
		$this->db->setQuery($query);
		$this->db->query();
	}
	
	public function uninstall()
	{
		$cid = JRequest::getVar('cid');
		$id  = implode(",", $cid);
		
		$query = "SELECT E.extension_id FROM #__extensions AS E, #__zof_themecfg AS T 
		WHERE E.type= 'template' AND E.element = T.theme AND T.id IN ({$id})";
		$this->db->setQuery($query);
		$rows  = $this->db->loadResultArray();
		
		if(count($rows)) {
			foreach($rows as $id) {
				$installer 	= new JInstaller;
				$result 	= $installer->uninstall('template', $id);
			}
		}
		
		$app = JFactory::getApplication();
		$msg = ($result) ? $app->enqueueMessage(JTEXT::_('Uninstalling template(s) was successful !')) : '';
		
		$app->setUserState('com_zo2framework.extension_message', $installer->get('extension_message'));
		
		return $result;
	}
}