<?php
/*
# ------------------------------------------------------------------------
# ZTTools plugin for Joomla 2.5
# ------------------------------------------------------------------------
# Copyright(C) 2008-2012 www.zootemplate.com. All Rights Reserved.
# @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
# Author: ZooTemplate
# Websites: http://www.zootemplate.com
# ------------------------------------------------------------------------
*/

defined('_JEXEC') or die();
jimport('joomla.plugin.plugin');
jimport('joomla.application.module.helper');
jimport('joomla.html.parameter');

require_once(dirname(__FILE__) . DS . 'plg_ztools' . DS . 'define.php');
require_once(dirname(__FILE__) . DS . 'plg_ztools' . DS . 'common.php');

class plgSystemPlg_ZTools extends JPlugin
{
	
	var $plugin 		= NULL;
	var $plgParams 		= NULL;
	var $time 			= 0;
	var $_cache 		= NULL;
	var $_components	= NULL;
	var $_db			= NULL;
	
	function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		
		$this->plugin = JPluginHelper::getPlugin('system', 'plg_ztools');
		$this->_db    = JFactory::getDBO();
		
		// Get params from component
		$query 	= "SELECT params FROM #__zof_config WHERE `id` = 1";
		$this->_db->setQuery($query);
		$params = $this->_db->loadResult();
		
		// Render params
		$this->plgParams 	= new JParameter($params);
		
		// Get exclude categories
		$components			= $this->plgParams->get('cache_components', '');
		$this->_components 	= (is_array($components)) ? $components : array($components);
		
		//Set the language in the class
		$config = JFactory::getConfig();
		
		$options = array(
			'cachebase'		=> JPATH_ROOT . DS . "cache" . DS . $this->plgParams->get('cache_folder', ''),
			'lifetime'		=> (int)($this->plgParams->get('cache_time', 15) * 60),
			'defaultgroup'	=> 'page',
			'browsercache'	=> ($this->plgParams->get('cache_enable', 0)) ? true : false,
			'caching'		=> false,
		);

		jimport('joomla.cache.cache');
		$this->_cache = JCache::getInstance('page', $options);
	}		
	
	function checkCurrentComp()
	{
		global $mainframe, $option;
				
		$option = JRequest::getVar('option');
		$view	= JRequest::getVar('view');
		
		// Includes component and view
		if($this->plgParams->get('cache_filter', 0)) {
			$return = false;
			if(in_array($option, $this->_components)) {
				$return = true;
			}
			if(in_array($option . '.' . $view, $this->_components)) {
				$return = true;
			}
			return $return;
		}
		// Excludes component and view
		else {
			$return = true;
			if(in_array($option, $this->_components)) {
				$return = false;
			}
			if(in_array($option . '.' . $view, $this->_components)) {
				$return = false;
			}
			return $return;
		}
	}
	
	function onAfterInitialise()
	{
		// Reload template
		$tpath = JPATH_ROOT . DS . "administrator" . DS . "components" . DS . "com_zo2framework" . DS . "models" . DS . "templates.php";
		
		if(is_file($tpath)) {
			require_once $tpath;
			$template = new Zo2FrameworkModelTemplates;
		}
		
		if($this->plgParams->get('cache_enable', 0))
		{
			global $_PROFILER;
			$app  = JFactory::getApplication();
			$user = JFactory::getUser();
	
			if($app->isAdmin() || JDEBUG){return;}
	
			if(!$user->get('guest') && $_SERVER['REQUEST_METHOD'] == 'GET' && $this->checkCurrentComp()) {
				$this->_cache->setCaching(true);
			}
			
			$data  = $this->_cache->get();
	
			if($data !== false)
			{
				JResponse::setBody($data);
				echo JResponse::toString($app->getCfg('gzip'));
	
				if(JDEBUG)
				{
					$_PROFILER->mark('afterCache');
					echo implode('', $_PROFILER->getBuffer());
				}
	
				$app->close();
			}
		}
	}
	
	// Add params for menu
	function onContentPrepareForm($form, $data)
	{
		if($form->getName() == 'com_menus.item')
		{
			JForm::addFormPath(JPATH_SITE . DS . ZT_MEGA_MENU_PARAMS);
			$form->loadFile('params', false);
		}
	}
	
	function onAfterRender()
	{
		global $app;
		
		$document = &JFactory::getDocument();
		$option   = JRequest::getVar('option', '');
		$task	  = JRequest::getVar('task', '');
		
        ZTimport('plg_ztools.libs.ztgzip');
		$Gzip = new ZTGzip($this->plgParams);
                
        $type	= JRequest::getVar('type');
		$action = JRequest::getVar('action');
                
		if($app->isSite() && $document->_type == 'html' && !$app->getCfg('offline') 
		&& (!($option == 'com_content' && $task =='edit')))
		{	
			if($this->plgParams->get('optimize_lazyload', 0)) 	$Gzip->lazyload();
			if($this->plgParams->get('optimize_js', 0)) 		$Gzip->optimizejs();
			if($this->plgParams->get('optimize_css', 0)) 		$Gzip->optimizecss();
			if($this->plgParams->get('optimize_html', 0)) 		$Gzip->optimizehtml();
			
			if($this->plgParams->get('cache_enable', 0))
			{
				$user = JFactory::getUser();
				if(!$user->get('guest')) {
					$this->_cache->store();
				}
			}
		}
		else
		{
			$uri 	= JURI::root() . '/media/com_zo2framework/';
			$html 	= '<script language="javascript" type="text/javascript" src="'.$uri.'/js/backend/clear.cache.js"></script>';
			
			$buffer = JResponse::getBody();
			$buffer = preg_replace('/<\/head>/', $html . "\n</head>", $buffer);
			
			JResponse::setBody($buffer);						
		}

        $buffer = JResponse::getBody();
        $buffer = preg_replace('/<meta name="og:/', '<meta property="og:', $buffer);
        $buffer = preg_replace('/<meta name="title/', '<meta property="title', $buffer);
        JResponse::setBody($buffer);

		if($type == 'plugin' && $action == 'clearCache' && $app->isSite()) {
			$Gzip->clearCache();
		}
	}
	
}